//
// ET++ Tutorial  Lernabschnitt: 2   Uebung: hello6
//
//
// Hello world with ET++:
// - the string "hello world" is shown in a scrollable window
// - the string "bonjour tout le monde" is shown in a second window with a splitter
// - add second view to each window by means of expander
// - the current layout can be loaded from a file
// - the titletext is read from document file
//   the file provided is: hellotext5,  hellotext5orig
//

#ifdef ET_DUMP
#include "ET_Dump.h"
#endif

#include "ET++.h"
#include "Collection.h"

//---- HelloView -------------------------------------------------------------------

class HelloView: public View { 
    TextItem *viewtext;
public:
    MetaDef(HelloView);
    HelloView(Document *dp, Rectangle itsExtent) : View(dp, itsExtent)
	{ }
    void Draw(Rectangle r);
    void SetText(TextItem *t);
};

MetaImpl(HelloView, (TP(viewtext)));
	    
void HelloView::SetText(TextItem *t)
{
    viewtext = t;
    // force the the text item to calculate its extent 
    viewtext->CalcExtent(); 
    // install the text item in this view
    viewtext->SetContainer(this);
    ForceRedraw();
}

void HelloView::Draw(Rectangle r)
{ 
    viewtext->DrawAll(r); 
}



//---- HelloDocument ---------------------------------------------------------------

class HelloDocument : public Document {
    HelloView *view1;
    HelloView *view2;
    TextItem  *text1;
    TextItem  *text2;
    TextItem  *windowtitle1;
    TextItem  *windowtitle2;
    Window    *mywin1;
    Window    *mywin2;

public:
    MetaDef(HelloDocument);
    HelloDocument();
    ~HelloDocument();
    Window *DoMakeWindows();
    void DoRead(istream &, class FileType *);
    void DoWrite(ostream &, int);
};

MetaImpl(HelloDocument, (TP(view1), TP(view2), TP(text1), TP(text2), TP(windowtitle1), TP(windowtitle2),
                         TP(mywin1), TP(mywin2)));

HelloDocument::HelloDocument() : Document("HELLO5")
{
    static int family= 1;

    family= (family+1) % 10;  // cycle through first 10 font families 
    text1= new TextItem("hello world            ",
	    new_Font(GrFont(family), 24, GrFace(eFaceBold|eFaceShadow))); 
    text1->SetOrigin(Point(50));
    text2= new TextItem(" bonjour tout le monde ",
	    new_Font(GrFont(family), 24, GrFace(eFaceBold|eFaceShadow))); 
    text2->SetOrigin(Point(50));
    
    windowtitle1= new TextItem(" english-francais ",
	    new_Font(GrFont(family), 24, GrFace(eFaceBold|eFaceShadow))); 
    windowtitle2= new TextItem(" francais-english ",
	    new_Font(GrFont(family), 24, GrFace(eFaceBold|eFaceShadow))); 
}

HelloDocument::~HelloDocument()
{
    SafeDelete(view1);
    SafeDelete(view2);
    SafeDelete(text1);
    SafeDelete(text2);
    SafeDelete(windowtitle1);
    SafeDelete(windowtitle2);
}

Window *HelloDocument::DoMakeWindows()
{
    view1= new HelloView(this, Point(600));
    view1->SetText(text1);
    view2= new HelloView(this, Point(600));
    view2->SetText(text2);

    Expander *exp1= new Expander(cIdNone, eHor, gPoint8, NULL);
    Expander *exp2= new Expander(cIdNone, eHor, gPoint8, NULL);
    exp1->Add(new Splitter(view1));
    exp1->Add(new Scroller(view2));
    exp2->Add(new Scroller(view2));
    exp2->Add(new Scroller(view1));

    mywin1 = new Window(this, Point(400,400), eWinDefault, exp1);
    mywin2 = new Window(this, Point(1000,250), eWinDefault, exp2);
    mywin2->OpenAt( Point(10));
    AddWindow (mywin2);
    mywin1->SetTitle(windowtitle1->AsString(), TRUE);
    mywin2->SetTitle(windowtitle2->AsString(), TRUE);

    return mywin1;
 }


void HelloDocument::DoRead(istream &is, class FileType *ft)
{
    Document::DoRead(is, ft);    // read file type information
    SafeDelete(text1);
    SafeDelete(text2);
    SafeDelete(windowtitle1);
    SafeDelete(windowtitle2);
    is >> text1 >> text2 >> windowtitle1 >> windowtitle2;
    view1->SetText(text1);
    view2->SetText(text2);
    mywin1->SetTitle(windowtitle1->AsString(), TRUE);
    mywin2->SetTitle(windowtitle2->AsString(), TRUE);
}

void HelloDocument::DoWrite(ostream &os, int flag)
{
    Document::DoWrite(os, flag);  // store file type information
    os << text1 << text2 << windowtitle1 << windowtitle2;
}

//---- HelloApplication -------------------------------------------------------------------

class HelloApplication: public Application {
public:
    MetaDef(HelloApplication);
    HelloApplication(int argc, char **argv) : Application(argc, argv, "HELLO5")
	{ ApplInit(); }
    Document *DoMakeDocuments(Symbol)
	{ return new HelloDocument; }
};

MetaImpl0(HelloApplication);

//---- main --------------------------------------------------------------------

main(int argc, char **argv)
{
    return HelloApplication(argc, argv).Run();
}
