
#include "ET++.h"

#include "ShapeView.h"
#include "ShapeCommand.h"

extern int cMAXINKS;

//---- ShapeView ---------------------------------------------------------------

MetaImpl(ShapeView, (TP(shape1), TP(shape2), TP(selected), TP(dialog)));

ShapeView::ShapeView(Document *d, Point ext) : View(d, ext)
{   
    shape1 = new BoxShape(Rectangle(100,100,100,100));
    shape2 = new OvalShape(Rectangle(150,150,100,100));
    shape1->SetContainer(this);
    shape2->SetContainer(this);

    dialog = new ShapeDialog("Shape Options");
}

ShapeView::~ShapeView()
{
    SafeDelete(shape1);
    SafeDelete(shape2);

    SafeDelete(dialog);
}

void ShapeView::Draw(Rectangle r)
{
    shape1->DrawAll(r);
    shape2->DrawAll(r);
}


Command *ShapeView::DoLeftButtonDownCommand(Point p, Token t, int cl)
{
    if (shape2->ContainsPoint(p))
        selected= shape2;
    else if (shape1->ContainsPoint(p))
        selected= shape1;
    if (selected) {
        if (t.Flags & eFlgCntlKey)
            return selected->GetStretcher();
        return selected->GetMover();
    }

    return View::DoLeftButtonDownCommand(p, t, cl);
}


istream &ShapeView::ReadFrom(istream &s)
{
    SafeDelete(shape1);
    SafeDelete(shape2);
    s >> shape1 >> shape2;
    shape1->SetContainer(this);
    shape2->SetContainer(this);
    ForceRedraw();
    return s;
}


void ShapeView::DoCreateMenu(Menu *menu)
{
    Menu *submenu= new Menu("patterns", FALSE, 0, 2);

    for (int i= 0; palette[i]; i++)
	submenu->Append(new PatternMenuItem(i));
    
    View::DoCreateMenu(menu);
    menu->AppendMenu(submenu, cPATTERNMENU);
}

void ShapeView::DoSetupMenu(Menu *menu)
{
    View::DoSetupMenu(menu);
    for (int p= 0; palette[p]; p++)
	menu->EnableItem(cSETPATTERN+p);
    menu->EnableItems(cPATTERNMENU, cPASTE, 0);
}

Command *ShapeView::DoMenuCommand(int cmd)
{
    if (cmd >= cSETPATTERN && cmd <= cSETPATTERN+cMAXINKS && selected)
	return new PatternCommand(selected, cmd-cSETPATTERN);
    return View::DoMenuCommand(cmd);
}


Command *ShapeView::DoMiddleButtonDownCommand(Point p, Token t, int)
{
    selected= 0;

    if (shape2->ContainsPoint(p))
        selected= shape2;
    else if (shape1->ContainsPoint(p))
        selected= shape1;

    if (selected) {
    	dialog->Show(selected);
    }
    return gNoChanges;
}


